# 移动端工程模板

## 目录

- [项目目录结构](#项目目录结构)
- [可用脚本](#可用脚本)
  - [npm start](#npm-start)
  - [npm run lint](#npm-run-lint)
  - [npm run lint-css](#npm-run-lint-css)
  - [npm run build](#npm-run-build)
- [引入图片和文件](#引入图片和文件)
- [public文件夹的使用](#public文件夹的使用)
  - [引入外部资源](#引入外部资源)
- [添加路由](#添加路由)
  - [路由跳转](#路由跳转)
- [引用样式](#引用样式)
- [引用 weui 组件](#引用-weui-组件)
- [国际化](#国际化)
- [模板生成](#模板生成)
  - [生成组件](#生成组件)
  - [生成路由](#生成路由)
  - [生成action](#生成action)
  - [生成reducer](#生成reducer)
- [通过fetch获取数据](#通过fetch获取数据)

## 项目目录结构

下载模板后，将看到以下项目文件夹结构：

```
└───wlk-react-template/
  ├───package.json // node相关环境的配置文件
  └───src/
    ├───actions    // 请求处理
    ├───config     // 接口地址配置
    ├───components // 页面组件
    ├───locales    // 国际化信息
    ├───reducers   // 存放state
    ├───routes     // 组件路由
    ├───utils      // 提供一些小工具
    ├───App.js     // 根react组件
    ├───app.less   // 公共样式
    ├───entry.js   // webpack打包入口
    ├───i18n.js    // 国际化处理
    ├───index.html // html模板文件
  └───test         // jest测试集
    ├───__snapshots__
  ├───.eslintrc    // eslint代码检查配置
  ├───.stylelintrc // css语法检测配置
  └───webpack.config.js // webpack配置文件
```

当项目构建时，**必须包含以下文件**：

- `src/index.html` 是页面模板
- `src/entry.js` 是入口文件

可以在 `src` 目录创建子目录，为了更快地重新构建，Webpack 只处理 `src` 中的文件。**需要将 JS 和 CSS 放到 `src` 里面**，否则 Webpack 将不会处理。

### 引入图片和文件

使用静态模块，比如图片和样式等，通过 Webpack 编译。

可以通过**`import` 引入 JavaScript 模块**，使 Webpack 打包时包含这些文件。
这些文件的路径最终将在 html 自动引用，比如通过图片的 `src` 属性或者通过 `href` 引入样式。

为了减少页面请求，导入的图片小于 1000 bytes 将返回一个[data URI](https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/Data_URIs) 替换图片地址。支持格式：bmp, gif, jpg, jpeg, 和 png。

以下有个例子：

```js
import React from "react";
import logo from "./logo.png"; // Tell Webpack this JS file uses this image

console.log(logo); // /logo.84287d09.png

function Header() {
  // Import result is the URL of your image
  return <img src={logo} alt="Logo" />;
}

export default Header;
```

当项目构建完，Webpack 会将图片放到 `build` 目录中，以及引用正确的访问地址。

同样适合在 CSS 中：

```css
.Logo {
  background-image: url(./logo.png);
}
```

### public文件夹的使用

#### `引入外部资源`

可以通过`public` 文件夹引入其他模块代码。

注意，建议使用 JavaScript 文件中的`import`引入。
例如，请参阅[样式引用](#样式引用)和[引入图片和文件](#引入图片和文件)部分。

这种机制提供了许多好处:

- 脚本和样式表被压缩并打包在一起，以避免额外的网络请求。
- 丢失的文件会导致编译错误，而不会造成 404 错误。

如果将一个文件放入 `public` 文件夹，会被 Webpack 忽略，并被复制到 build 文件夹中。

要引用 `public` 文件夹中的文件，需要使用一个名为 `PUBLIC_URL` 的特殊变量。

在 `index.html` 里面，也可以这么使用：

```html
<link rel="shortcut icon" href="%PUBLIC_URL%/favicon.ico">
```

当执行 `npm run build`，模板将会替换正确的路径。


```js
render() {
  // Note: this is an escape hatch and should be used sparingly!
  // Normally we recommend using `import` for getting asset URLs
  // as described in “Adding Images and Fonts” above this section.
  return <img src={process.env.PUBLIC_URL + '/img/logo.png'} />;
}
```

这种方法有以下缺点:

- `public` 文件夹内的所有文件都不会被处理或压缩。
- 在编译时将不会因为丢失的文件或 js 语法问题，而提示错误。

#### 什么时候适合使用 public 文件夹

`public`文件夹对于一些不太常见的情况是有用的:

- 有数千个图像，需要动态地引用它们的路径。
- 想要引入一段脚本比如[`pack.js`]而不经过 webpack 打包编译。
- 有些库可能与 Webpack 不兼容，没有其他选择，只能将它通过`<script>`标签引入。

### 添加路由

在 `src/app.json` 添加配置，如下：

```json
{
  "pages": {
    "./Home/index": "/",
    "./Desc/index": "/desc"
  }
}
```

以键值对存在：

- `key` 表示 `routes` 目录下具体路由组件文件的位置，不需要带文件后缀.js，如 `./Home/index`；

- `value` 表示页面访问的 hash 路由。如首页配置 `/`，访问地址为：`html/index.html#/`。又如 desc页面配置 `/desc`，访问地址为：`html/index.html#/desc`。

#### `路由跳转`

方法一：使用 Link 组件进行跳转

```js
import { Link } from "react-router-dom";
<Link to="/desc" className="desc-link">
  使用说明
</Link>;
```

方法二：使用 history 进行跳转

```js
import createHistory from "history/createHashHistory";
const history = createHistory();
history.push("/desc"); // 跳转路由
history.goForward(); // 访问前一个页面
history.goBack(); // 后退
```

### 样式引用

项目使用[Webpack](https://webpack.js.org/) 对所有模块进行处理。
Webpack 提供了一种自定义的方式来“扩展”，通过 JavaScript 的`import`，
在 JavaScript 中引用 CSS 文件:

#### `Button.css`

```css
.Button {
  padding: 20px;
}
```

#### `Button.js`

```js
import React, { Component } from "react";
import "./Button.css"; // Tell Webpack that Button.js uses these styles

class Button extends Component {
  render() {
    // You can use them as regular CSS styles
    return <div className="Button" />;
  }
}
```

### 使用标准UI组件

```jsx
import React from "react";
import {
  Tab,
  TabBody,
  TabBar,
  TabBarItem,
  TabBarIcon,
  TabBarLabel,
  Article
} from "@wecode/react-weui";
```


### 国际化

输出 src/locales 中 common.json 配置的国际化字段 appName 信息，国际化初始化判断请参考 `src/entry.js`

使用：

```jsx
import i18n from "i18n";
i18n.t("common:appName");
```

### 模板生成

通过命令行可生成 Action，Component，Reducer 和 Route 模板，代替人工新增或复制文件或目录，减少冗余操作，提高工作效率。

#### `生成组件`

```bash
  npm run add component Demo
```

执行完命令后，将在 `src/components` 下生成文件夹 `Demo`。

#### `生成路由`

```bash
  npm run add route demo
```

执行完命令后，将在 `src/routes` 下生成文件夹 `Demo`。

#### `生成action`

```bash
  npm run add route demo
```

执行完命令后，将在 `src/actions` 下生成文件 `demo.js`。

#### `生成reducer`

```bash
  npm run add reducer demo
```

执行完命令后，将在 `src/reducers` 下生成文件 `demo.js`。

## 通过fetch获取数据

**We码开发数据请求只能通过 fetchInternet。**

在开发环境，`window.HWH5.fetchInternet` 使用的是浏览器的fetch能力。但在We码中，利用的是原生能力去请求。
